MODULE compute_caldyn_mod
  USE prec, ONLY : rstd
  IMPLICIT NONE
  
  ! fake array dimensions, for interfaces
  INTEGER, PARAMETER :: iim_jjm_i=1, iim_jjm_u=1, iim_jjm_v =1, llm_=1, llm1=1, nqdyn_=1

  INTERFACE
     
    SUBROUTINE comp_pvort_only(u,rhodz,qu,qv, hv)
      IMPORT
      REAL(rstd), INTENT(IN)    :: u(:,:)
      REAL(rstd), INTENT(INOUT) :: rhodz(:,:)
      REAL(rstd), INTENT(OUT)   :: qu(:,:)
      REAL(rstd), INTENT(OUT)   :: qv(:,:)
      REAL(rstd), INTENT(OUT)   :: hv(:,:)
    END SUBROUTINE comp_pvort_only

    SUBROUTINE comp_theta(mass_col,theta_rhodz, rhodz,theta)
      IMPORT
      REAL(rstd), INTENT(IN)    :: mass_col(:)
      REAL(rstd), INTENT(IN)    :: theta_rhodz(:,:,:)
      REAL(rstd), INTENT(INOUT) :: rhodz(:,:)
      REAL(rstd), INTENT(OUT)   :: theta(:,:,:)
    END SUBROUTINE comp_theta

    SUBROUTINE comp_geopot(rhodz,theta, ps,pk,geopot) 
      IMPORT
      REAL(rstd), INTENT(IN)    :: rhodz(:,:)    ! rho*dz = mass per unit surface in each full model level
      REAL(rstd), INTENT(IN)    :: theta(:,:,:)  ! active scalars : theta/entropy, moisture, ...
      REAL(rstd), INTENT(INOUT) :: ps(:)         ! surface pressure
      REAL(rstd), INTENT(OUT)   :: pk(:,:)       ! Exner function (compressible) /Lagrange multiplier (Boussinesq)
      REAL(rstd), INTENT(INOUT) :: geopot(:,:)   ! geopotential
    END SUBROUTINE comp_geopot

    SUBROUTINE comp_caldyn_fast(tau,theta,geopot, pk,berni,du,u)
      IMPORT
      REAL(rstd), INTENT(IN)    :: tau           ! "solve" u-tau*du/dt = rhs
      REAL(rstd), INTENT(IN)    :: theta(:,:,:)
      REAL(rstd), INTENT(IN)    :: geopot(:,:)
      REAL(rstd), INTENT(INOUT) :: pk(:,:)
      REAL(rstd), INTENT(INOUT) :: berni(:,:)    ! partial Bernoulli function
      REAL(rstd), INTENT(INOUT) :: du(:,:)
      REAL(rstd), INTENT(INOUT) :: u(:,:)        ! INOUT if tau>0
    END SUBROUTINE comp_caldyn_fast

    SUBROUTINE comp_caldyn_slow_hydro(zero, u,rhodz,hv,Kv, berni, hflux,du)
      IMPORT
      LOGICAL,    INTENT(IN)    :: zero
      REAL(rstd), INTENT(IN)    :: u(:,:)      ! prognostic "velocity"
      REAL(rstd), INTENT(IN)    :: rhodz(:,:)
      REAL(rstd), INTENT(IN)    :: hv(:,:)     ! height/mass averaged to vertices
      REAL(rstd), INTENT(IN)    :: Kv(:,:)     ! kinetic energy at vertices
      REAL(rstd), INTENT(OUT)   :: berni(:,:)  ! Bernoulli function
      REAL(rstd), INTENT(OUT)   :: hflux(:,:)  ! hflux in kg/s 
      REAL(rstd), INTENT(INOUT) :: du(:,:)
    END SUBROUTINE comp_caldyn_slow_hydro
    
    SUBROUTINE comp_caldyn_coriolis(hflux,theta,qu, Ftheta, convm,dtheta_rhodz,du)
      IMPORT
      REAL(rstd), INTENT(IN)    :: hflux(:,:)   ! hflux in kg/s
      REAL(rstd), INTENT(IN)    :: theta(:,:,:) ! active scalars
      REAL(rstd), INTENT(IN)    :: qu(:,:)
      REAL(rstd), INTENT(OUT)   :: Ftheta(:,:)  ! potential temperature flux
      REAL(rstd), INTENT(OUT)   :: convm(:,:)   ! mass flux convergence
      REAL(rstd), INTENT(OUT)   :: dtheta_rhodz(:,:,:)
      REAL(rstd), INTENT(INOUT) :: du(:,:)
    END SUBROUTINE comp_caldyn_coriolis

  END INTERFACE

  PROCEDURE(comp_pvort_only),        POINTER, SAVE :: compute_pvort_only        => NULL()
  PROCEDURE(comp_theta),             POINTER, SAVE :: compute_theta             => NULL()
  PROCEDURE(comp_geopot),            POINTER, SAVE :: compute_geopot            => NULL()
  PROCEDURE(comp_caldyn_fast),       POINTER, SAVE :: compute_caldyn_fast       => NULL()
  PROCEDURE(comp_caldyn_slow_hydro), POINTER, SAVE :: compute_caldyn_slow_hydro => NULL()
  PROCEDURE(comp_caldyn_coriolis),   POINTER, SAVE :: compute_caldyn_coriolis   => NULL()

END MODULE compute_caldyn_mod
